package PlayBot::sites::soundcloud;

use strict;
use warnings;
use utf8;

use LWP::UserAgent;
use JSON;
use Scalar::Util qw(looks_like_number);

use PlayBot::utils::Logging;

my $log = PlayBot::utils::Logging->new( 'STDOUT', 1 );

my $root     = 'https://api.soundcloud.com';
my $clientId = 'f4956716fe1a9dc9c3725af822963365';
our $clientSecret;
my $access_token = '';
my $access_token_time;

sub regex_playlist {
    return
qr#(?:^|[^!])https?://(?:www\.)?soundcloud.com/([a-zA-Z0-9_-]+/sets/[a-zA-Z0-9_-]+)(?:\?.+)?#;
}

sub regex {
    return
qr#(?:^|[^!])https?://(?:www\.)?soundcloud.com/([a-zA-Z0-9_-]+/[a-zA-Z0-9_-]+)(?:\?.+)?#;
}

sub get_access_token {
    if ( $access_token ne '' && time() - $access_token_time <= 3000 ) {
        return $access_token;
    }

    $log->info("Get a new soundcloud token");

    my $ua = LWP::UserAgent->new(
        timeout   => 30,
        env_proxy => 1,
    );
    my $response =
      $ua->post( $root
          . '/oauth2/token?grant_type=client_credentials&client_id='
          . $clientId
          . '&client_secret='
          . $clientSecret, );
    die( $response->status_line ) unless ( $response->is_success );

    my $content = decode_json( $response->decoded_content );
    $access_token      = $content->{'access_token'};
    $access_token_time = time();

    return $access_token;
}

sub get_playlist {
    shift;
    my $id = shift;

    my $ua = LWP::UserAgent->new(
        timeout   => 30,
        env_proxy => 1,
    );

    my $response;
    if ( looks_like_number($id) ) {
        $response =
          $ua->get( $root . '/playlists/' . $id . '?client_id=' . $clientId );
    }
    else {
        my $url = 'https://www.soundcloud.com/' . $id;
        $response = $ua->get(
            $root . '/resolve.json?url=' . $url . '&client_id=' . $clientId );
    }
    die( $response->status_line ) unless ( $response->is_success );

    my $content = decode_json( $response->decoded_content );
    my $infos   = {
        author      => $content->{'user'}->{'username'},
        external_id => $content->{'id'},
        duration    => $content->{'track_count'},
        site        => 'soundcloud',
        title       => $content->{'title'},
        url         => $content->{permalink_url},
    };

    my @urls;
    foreach my $item ( @{ $content->{'tracks'} } ) {
        push @urls, $item->{'permalink_url'};
    }

    $infos->{'urls'} = \@urls;
    return %$infos;
}

sub get {
    shift;
    my $id = shift;

    my $response;
    $log->info( "the id is " . $id );

    my $content = `./soundcloud $id`;
    my ( $external_id, $url, $title, $author, $duration ) =
      split( /\n/, $content );

=cut
	my $ua = LWP::UserAgent->new(
		timeout => 30,
		env_proxy   => 1,
	);

	if (looks_like_number($id))
	{
		$response = $ua->get(
			$root.'/tracks/'.$id,
			'Authorization' => 'OAuth '.get_access_token(),
		);
	}
	else
	{
		my $url = 'https://soundcloud.com/'.$id;
		$response = $ua->get(
			$root.'/resolve?url='.$url,
			'Authorization' => 'OAuth '.get_access_token(),
		);
	}

	die($response->status_line) unless ($response->is_success);

	my $content = decode_json($response->decoded_content);
	my $infos = {
		author      => $content->{'user'}->{'username'},
		external_id => $content->{'id'},
		duration    => int(0.5 + $content->{'duration'} / 1000),
		site        => 'soundcloud',
		title       => $content->{'title'},
		url         => $content->{permalink_url},
	};

	if ($content->{'downloadable'}) {
		$infos->{'ddl'} = $content->{'download_url'};
	}
=cut

    my $infos = {
        author      => $author,
        external_id => $external_id,
        duration    => $duration,
        site        => 'soundcloud',
        title       => $title,
        url         => $url,
    };

    return %$infos;
}

1;
